use super::spi_interface::{SpiMouseSensor, SpiPins};

use cortex_m::delay::Delay;
use embedded_hal::digital::v2::OutputPin;
use fugit::HertzU32;
use rp_pico::{
    hal::gpio::{self, bank0::Gpio25, Pin, PushPullOutput},
    pac::{RESETS, SPI0},
    Pins,
};

pub struct MouseSensor<L>
where
    L: FnMut(&str),
{
    pub spi_device: SpiMouseSensor<L>,

    led: Pin<Gpio25, PushPullOutput>,
}

impl<L> MouseSensor<L>
where
    L: FnMut(&str),
{
    pub fn new(
        pins: Pins,
        resets: &mut RESETS,
        spio: SPI0,
        peripheral_clock: impl Into<HertzU32>,
        delay: Delay,
        log: L,
    ) -> Self {
        // These are implicitly used by the spi driver if they are in the correct mode

        let spi_pins = SpiPins {
            sclk: pins.gpio2.into_mode::<gpio::FunctionSpi>(),
            mosi: pins.gpio3.into_mode::<gpio::FunctionSpi>(),
            miso: pins.gpio4.into_mode::<gpio::FunctionSpi>(),
            cs: pins.gpio5.into_push_pull_output(),
        };

        let led = pins.led.into_push_pull_output();
        let spi_device = SpiMouseSensor::new(spi_pins, resets, spio, peripheral_clock, delay, log);

        let mut me = Self { spi_device, led };

        if me.led.set_high().is_err() {
            (me.spi_device.log)("led high failed");
        }

        // verify initialization
        loop {
            if me.verify_product_id_1() {
                break;
            }
        }

        loop {
            if me.verify_product_id_2() {
                break;
            }
        }

        if me.led.set_low().is_err() {
            (me.spi_device.log)("led low failed");
        }

        me
    }

    pub fn verify_product_id_1(&mut self) -> bool {
        const DEFAULT_VALUE: u8 = 0x30;

        DEFAULT_VALUE == self.spi_device.product_id_1()
    }

    pub fn verify_product_id_2(&mut self) -> bool {
        const DEFAULT_VALUE: u8 = 0x02;

        DEFAULT_VALUE == self.spi_device.product_id_2()
    }

    fn motion_detected(&mut self) -> (bool, bool, bool) {
        let reg = self.spi_device.motion_status();

        let motion = (reg & 0b10000000) != 0;
        let x_overflow = (reg & 0b00010000) != 0;
        let y_overflow = (reg & 0b00001000) != 0;

        (motion, x_overflow, y_overflow)
    }

    pub fn read_movement(&mut self) -> Option<(i8, i8)> {
        let (motion, x_overflow, y_overflow) = self.motion_detected();

        if motion {
            if x_overflow || y_overflow {
                return None;
            }

            let x = self.spi_device.delta_x();
            let y = self.spi_device.delta_y();

            Some((x, y))
        } else {
            None
        }
    }
}
